use std::time::Duration;

/// Structure representing a Cookie
/// # Creating a Cookie:
/// ```
/// use http::handling::response::Cookie;
/// use http::handling::response::CookieBuilder;
///
/// let cookie = CookieBuilder::build("name", "value").finish();
/// ```
pub struct Cookie {
    /// Storage for the cookie string. Only used if this structure was derived
    /// from a string that was subsequently parsed.
    pub(crate) cookie_string: Option<String>,
    pub(crate) name: String,
    pub(crate) value: String,
    // expires: Option<Tm>,
    pub(crate) max_age: Option<Duration>,
    /// The cookie's domain, if any.
    pub(crate) domain: Option<String>,
    /// The cookie's path domain, if any.
    pub(crate) path: Option<String>,
    /// Whether this cookie was marked Secure.
    pub(crate) secure: Option<bool>,
    /// Whether this cookie was marked HttpOnly.
    pub(crate) http_only: Option<bool>,
    /// The draft `SameSite` attribute.
    pub(crate) same_site: Option<SameSite>,
    pub(crate) expires: Option<String>,
    pub(crate) partitioned: Option<bool>,
}

#[derive(Debug)]
/// SameSite Paremeters
pub enum SameSite {
    /// Requires Secure
    None,
    Lax,
    Strict,
}

impl std::fmt::Display for SameSite {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Self::None => write!(f, "SameSite=None; Secure"),
            Self::Lax => write!(f, "SameSite=Lax"),
            Self::Strict => write!(f, "SameSite=Strict"),
        }
    }
}

impl std::fmt::Display for Cookie {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        todo!()
    }
}
