use std::io::Result;

use tokio::{io::AsyncWriteExt, net::TcpStream};

use crate::handling::{methods::Method, request::Request, response::Status};

use super::Response;

impl Response {
    /// Builds a [`Vec<u8>`] valid http response from a [Response] and consumes it. Optionally
    /// takes in a request for things like [Method::Head]
    pub fn build(self, request: Option<Request>) -> Vec<u8> {
        let compiled_headers = format!(
            "HTTP/1.1 {}\r\nContent-Length: {}\r\nContent-Type: {}\r\n",
            self.status.unwrap_or(Status::Ok),
            self.body.get_len(),
            self.body.get_mime()
        ) + &self.headers.join("\r\n")
            + "\r\n";
        let is_head = if let Some(req) = request {
            req.method == Method::Head
        } else {
            false
        };
        let compiled_body = if is_head {
            vec![]
        } else {
            self.body.get_data()
        };
        let mut compiled_out: Vec<u8> = compiled_headers.into();
        compiled_out.extend_from_slice(&compiled_body);
        compiled_out
    }
    /// Builds and writes The http-Response, consumes the  [tokio::net::TcpStream] [Request] and [Response]
    pub async fn write(self, mut stream: TcpStream, request: Option<Request>) -> Result<()> {
        let resp = self.build(request);
        stream.write_all(&resp).await?;
        Ok(())
    }
}
